@@ -2,6 +2,12 @@
 # coding=utf-8
 # $Id$
 # ------------------------------------------------------------------------
+"""
+Default settings for FeinCMS
+
+All of these can be overridden by specifying them in the standard
+``settings.py`` file.
+"""
 
 from os.path import join
 
@@ -11,60 +17,76 @@ from django.conf import settings
 # ------------------------------------------------------------------------
 # Settings for MediaLibrary
 
-# Local path to uploaded media files
-FEINCMS_MEDIALIBRARY_ROOT = getattr(settings, 'FEINCMS_MEDIALIBRARY_ROOT', settings.MEDIA_ROOT)
-# Local path to newly uploaded media files
+#: Local path to newly uploaded media files
 FEINCMS_MEDIALIBRARY_UPLOAD_TO = getattr(settings, 'FEINCMS_MEDIALIBRARY_UPLOAD_TO', 'medialibrary/%Y/%m/')
-# URL to access media library files
-FEINCMS_MEDIALIBRARY_URL = getattr(settings, 'FEINCMS_MEDIALIBRARY_URL', settings.MEDIA_URL)
 
 # ------------------------------------------------------------------------
 # Settings for RichText
-FEINCMS_TINYMCE_INIT_TEMPLATE = 'admin/content/richtext/init_tinymce.html'
-FEINCMS_TINYMCE_INIT_CONTEXT  = {
+
+FEINCMS_RICHTEXT_INIT_TEMPLATE = getattr(settings, 'FEINCMS_RICHTEXT_INIT_TEMPLATE',
+    'admin/content/richtext/init_tinymce.html')
+FEINCMS_RICHTEXT_INIT_CONTEXT = getattr(settings, 'FEINCMS_RICHTEXT_INIT_CONTEXT', {
     'TINYMCE_JS_URL': join(settings.MEDIA_URL, 'js/tiny_mce/tiny_mce.js'),
     'TINYMCE_CONTENT_CSS_URL': None,
     'TINYMCE_LINK_LIST_URL': None
-}
-
-FEINCMS_RICHTEXT_INIT_TEMPLATE = getattr(settings, 'FEINCMS_RICHTEXT_INIT_TEMPLATE', FEINCMS_TINYMCE_INIT_TEMPLATE)
-FEINCMS_RICHTEXT_INIT_CONTEXT = getattr(settings, 'FEINCMS_RICHTEXT_INIT_CONTEXT', FEINCMS_TINYMCE_INIT_CONTEXT)
+    })
 
 # ------------------------------------------------------------------------
-# Admin settings
+# Admin media settings
 
-FEINCMS_ADMIN_MEDIA = getattr(settings, 'FEINCMS_ADMIN_MEDIA', '/media/sys/feincms/')
-# Link to google APIs instead of using local copy of JS libraries
+#: Path to FeinCMS' admin media
+FEINCMS_ADMIN_MEDIA = getattr(settings, 'FEINCMS_ADMIN_MEDIA', '/static/feincms/')
+#: Link to google APIs instead of using local copy of JS libraries
 FEINCMS_ADMIN_MEDIA_HOTLINKING = getattr(settings, 'FEINCMS_ADMIN_MEDIA_HOTLINKING', False)
-# avoid jQuery conflicts -- scripts should use feincms.jQuery instead of $
+#: avoid jQuery conflicts -- scripts should use feincms.jQuery instead of $
 FEINCMS_JQUERY_NO_CONFLICT = \
     getattr(settings, 'FEINCMS_JQUERY_NO_CONFLICT', False)
 
+# Django's admin media files have been transitioned over to standard staticfiles.
+# ADMIN_MEDIA_PREFIX isn't available anymore with newer versions (Django >= 1.4).
+if hasattr(settings, 'ADMIN_MEDIA_PREFIX'):
+    _HACK_ADMIN_MEDIA_IMAGES = settings.ADMIN_MEDIA_PREFIX + 'img/admin/'
+else:
+    _HACK_ADMIN_MEDIA_IMAGES = settings.STATIC_URL + 'admin/img/'
+
 # ------------------------------------------------------------------------
 # Settings for the page module
 
+#: Include ancestors in filtered tree editor lists
 FEINCMS_TREE_EDITOR_INCLUDE_ANCESTORS = getattr(settings, 'FEINCMS_TREE_EDITOR_INCLUDE_ANCESTORS', False)
 
-# ------------------------------------------------------------------------
-# Enable caching intermediate results in feincms. Be aware that this might deliver
-# slightly out of date pages if you are not using the 'changedate' page extension.
+#: Show frontend-editing button?
+FEINCMS_FRONTEND_EDITING = getattr(settings, 'FEINCMS_FRONTEND_EDITING', True)
 
+#: Enable checking of object level permissions. Note that if this option is enabled,
+#: you must plug in an authentication backend that actually does implement object
+#: level permissions or no page will be editable.
+FEINCMS_TREE_EDITOR_OBJECT_PERMISSIONS = getattr(settings, 'FEINCMS_TREE_EDITOR_OBJECT_PERMISSIONS', False)
+
+# ------------------------------------------------------------------------
+# Various settings
+
+# ------------------------------------------------------------------------
+#: Enable caching intermediate results in feincms. Be aware that this might deliver
+#: slightly out of date pages if you are not using the 'changedate' page extension.
 FEINCMS_USE_CACHE = getattr(settings, 'FEINCMS_USE_CACHE', False)
 
 # ------------------------------------------------------------------------
-# Allow random gunk after a valid (non appcontent) page?
-
+#: Allow random gunk after a valid page?
 FEINCMS_ALLOW_EXTRA_PATH = getattr(settings, 'FEINCMS_ALLOW_EXTRA_PATH', False)
 
 # ------------------------------------------------------------------------
-# How to switch languages.
-#   STANDARD = the page a user navigates to sets the site's language and overwrites
-#              whatever was set before
-#   EXPLICIT = the language set has priority, may only be overridden by explicitely
-#              setting a language with ?set_language=xx
-
+#: How to switch languages.
+#:   STANDARD = the page a user navigates to sets the site's language and overwrites
+#:              whatever was set before
+#:   EXPLICIT = the language set has priority, may only be overridden by explicitely
+#:              setting a language with ?set_language=xx
 FEINCMS_TRANSLATION_POLICY = getattr(settings, 'FEINCMS_TRANSLATION_POLICY', 'STANDARD')
 
+# ------------------------------------------------------------------------
+#: Set to True if you want to run the FeinCMS test suite unconditionally:
+FEINCMS_RUN_TESTS = getattr(settings, 'FEINCMS_RUN_TESTS', False)
+
 # ------------------------------------------------------------------------
 # Settings for HTML validation
 
@@ -76,7 +98,3 @@ FEINCMS_TIDY_SHOW_WARNINGS = getattr(settings, 'FEINCMS_TIDY_SHOW_WARNINGS', Tru
 FEINCMS_TIDY_ALLOW_WARNINGS_OVERRIDE = getattr(settings, 'FEINCMS_TIDY_ALLOW_WARNINGS_OVERRIDE', True)
 #: Name of the tidy function - anything which takes (html) and returns (html, errors, warnings) can be used:
 FEINCMS_TIDY_FUNCTION = getattr(settings, 'FEINCMS_TIDY_FUNCTION', 'feincms.utils.html.tidy.tidy_html')
-
-# ------------------------------------------------------------------------
-#: Set to True if you want to run the FeinCMS test suite unconditionally:
-FEINCMS_RUN_TESTS = getattr(settings, 'FEINCMS_RUN_TESTS', False)
