@@ -2,17 +2,20 @@
 This is the core of FeinCMS
 
 All models defined here are abstract, which means no tables are created in
-the feincms_ namespace.
+the feincms\_ namespace.
 """
 
+import itertools
+import operator
 import warnings
 
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ImproperlyConfigured
-from django.db import models
+from django.db import connection, models
 from django.db.models import Q
 from django.db.models.fields import FieldDoesNotExist
 from django.db.models.loading import get_model
+from django.forms.widgets import Media
 from django.template.loader import render_to_string
 from django.utils.datastructures import SortedDict
 from django.utils.encoding import force_unicode
@@ -21,6 +24,11 @@ from django.utils.translation import ugettext_lazy as _
 from feincms import settings, ensure_completely_loaded
 from feincms.utils import get_object, copy_model_instance
 
+try:
+    import reversion
+except ImportError:
+    reversion = None
+
 try:
     any
 except NameError:
@@ -84,15 +92,170 @@ class Template(object):
 
 class ContentProxy(object):
     """
-    This proxy offers attribute-style access to the page contents of regions::
+    The ``ContentProxy`` is responsible for loading the content blocks for all
+    regions (including content blocks in inherited regions) and assembling media
+    definitions.
 
-        >> page = Page.objects.all()[0]
-        >> page.content.main
-        [A list of all page contents which are assigned to the region with key 'main']
+    The content inside a region can be fetched using attribute access with
+    the region key. This is achieved through a custom ``__getattr__``
+    implementation.
     """
 
     def __init__(self, item):
+        item._needs_content_types()
         self.item = item
+        self._cache = {
+            'cts': {},
+            }
+
+    def _inherit_from(self):
+        """
+        Returns a list of item primary keys to try inheriting content from if
+        a certain region is empty.
+
+        Returns an ascending list of ancestors (using MPTT) by default. This
+        is good enough (tm) for pages.
+        """
+
+        return self.item.get_ancestors(ascending=True).values_list('pk', flat=True)
+
+    def _fetch_content_type_counts(self):
+        """
+        Returns a structure describing which content types exist for the object
+        with the given primary key.
+
+        The structure is as follows, where pk is the passed primary key and
+        ct_idx are indices into the item._feincms_content_types list:
+
+            {
+                'region_key': [
+                    (pk, ct_idx1),
+                    (pk, ct_idx2),
+                    ],
+                ...
+            }
+        """
+
+        if 'counts' not in self._cache:
+            counts = self._fetch_content_type_count_helper(self.item.pk)
+
+            empty_inherited_regions = set()
+            for region in self.item.template.regions:
+                if region.inherited and not counts.get(region.key):
+                    empty_inherited_regions.add(region.key)
+
+            if empty_inherited_regions:
+                for parent in self._inherit_from():
+                    parent_counts = self._fetch_content_type_count_helper(
+                        parent, regions=tuple(empty_inherited_regions))
+
+                    counts.update(parent_counts)
+                    for key in parent_counts.keys():
+                        empty_inherited_regions.discard(key)
+
+                    if not empty_inherited_regions:
+                        break
+
+            self._cache['counts'] = counts
+        return self._cache['counts']
+
+    def _fetch_content_type_count_helper(self, pk, regions=None):
+        tmpl = ['SELECT %d AS ct_idx, region, COUNT(id) FROM %s WHERE parent_id=%s']
+        args = []
+
+        if regions:
+            tmpl.append('AND region IN (' + ','.join(['%%s'] * len(regions)) + ')')
+            args.extend(regions * len(self.item._feincms_content_types))
+
+        tmpl.append('GROUP BY region')
+        tmpl = u' '.join(tmpl)
+
+        sql = ' UNION '.join([tmpl % (idx, cls._meta.db_table, pk)\
+            for idx, cls in enumerate(self.item._feincms_content_types)])
+        sql = 'SELECT * FROM ( ' + sql + ' ) AS ct ORDER BY ct_idx'
+
+        cursor = connection.cursor()
+        cursor.execute(sql, args)
+
+        _c = {}
+        for ct_idx, region, count in cursor.fetchall():
+            if count:
+                _c.setdefault(region, []).append((pk, ct_idx))
+
+        return _c
+
+    def _popuplate_content_type_caches(self, types):
+        """
+        Populate internal caches for all content types passed
+        """
+
+        counts_by_type = {}
+        for region, counts in self._fetch_content_type_counts().items():
+            for pk, ct_idx in counts:
+                counts_by_type.setdefault(self.item._feincms_content_types[ct_idx], []).append((region, pk))
+
+        # Resolve abstract to concrete content types
+        types = tuple(types)
+
+        for type in (type for type in self.item._feincms_content_types if issubclass(type, types)):
+            counts = counts_by_type.get(type)
+            if type not in self._cache['cts']:
+                if counts:
+                    self._cache['cts'][type] = list(type.get_queryset(
+                        reduce(operator.or_, (Q(region=r[0], parent=r[1]) for r in counts))))
+                else:
+                    self._cache['cts'][type] = []
+
+    def _fetch_regions(self):
+        """
+        """
+
+        if 'regions' not in self._cache:
+            self._popuplate_content_type_caches(self.item._feincms_content_types)
+            contents = {}
+            for type, content_list in self._cache['cts'].items():
+                for instance in content_list:
+                    contents.setdefault(instance.region, []).append(instance)
+
+            self._cache['regions'] = dict((region, sorted(instances, key=lambda c: c.ordering))\
+                for region, instances in contents.iteritems())
+        return self._cache['regions']
+
+    def all_of_type(self, type_or_tuple):
+        """
+        Return all content type instances belonging to the type or types passed.
+        If you want to filter for several types at the same time, type must be
+        a tuple.
+        """
+
+        content_list = []
+        if not hasattr(type_or_tuple, '__iter__'):
+            type_or_tuple = (type_or_tuple,)
+        self._popuplate_content_type_caches(type_or_tuple)
+
+        for type, contents in self._cache['cts'].items():
+            if any(issubclass(type, t) for t in type_or_tuple):
+                content_list.extend(contents)
+
+        # TODO: Sort content types by region?
+        return sorted(content_list, key=lambda c: c.ordering)
+
+    def _get_media(self):
+        """
+        Collect the media files of all content types of the current object
+        """
+
+        if 'media' not in self._cache:
+            media = Media()
+
+            for contents in self._fetch_regions().values():
+                for content in contents:
+                    if hasattr(content, 'media'):
+                        media = media + content.media
+
+            self._cache['media'] = media
+        return self._cache['media']
+    media = property(_get_media)
 
     def __getattr__(self, attr):
         """
@@ -105,40 +268,87 @@ class ContentProxy(object):
         if (attr.startswith('__')):
             raise AttributeError
 
-        item = self.__dict__['item']
-
-        return self.get_content(item, attr)
-
-    def get_content(self, item, attr):
-        template = item.template
-        try:
-            region = template.regions_dict[attr]
-        except KeyError:
+        # Do not trigger loading of real content type models if not necessary
+        if not self._fetch_content_type_counts().get(attr):
             return []
 
-        def collect_items(obj):
-            contents = obj._content_for_region(region)
+        return self._fetch_regions().get(attr, [])
 
-            # go to parent if this model has a parent attribute
-            # TODO: this should be abstracted into a property/method or something
-            # The link which should be followed is not always '.parent'
-            if region.inherited and not contents and hasattr(obj, 'parent_id') and obj.parent_id:
-                return collect_items(obj.parent)
 
-            return contents
+class ExtensionsMixin(object):
+    @classmethod
+    def register_extension(cls, register_fn):
+        """
+        Call the register function of an extension. You must override this
+        if you provide a custom ModelAdmin class and want your extensions to
+        be able to patch stuff in.
+        """
+        register_fn(cls, None)
 
-        contents = collect_items(item)
-        contents.sort(key=lambda c: c.ordering)
-        return contents
+    @classmethod
+    def register_extensions(cls, *extensions):
+        """
+        Register all extensions passed as arguments.
+
+        Extensions should be specified as a string to the python module
+        containing the extension. If it is a bundled extension of FeinCMS,
+        you do not need to specify the full python module path -- only
+        specifying the last part (f.e. ``'seo'`` or ``'translations'``) is
+        sufficient.
+        """
+
+        if not hasattr(cls, '_feincms_extensions'):
+            cls._feincms_extensions = set()
+
+        here = cls.__module__.split('.')[:-1]
+
+        paths = [
+            '.'.join(here + ['extensions']),
+            '.'.join(here[:-1] + ['extensions']),
+            'feincms.module.extensions',
+            ]
+
+        for ext in extensions:
+            if ext in cls._feincms_extensions:
+                continue
+
+            fn = None
+            if isinstance(ext, basestring):
+                try:
+                    fn = get_object(ext + '.register')
+                except ImportError:
+                    for path in paths:
+                        try:
+                            fn = get_object('%s.%s.register' % (path, ext))
+                            if fn:
+                                break
+                        except ImportError, e:
+                            pass
+
+                if not fn:
+                    raise ImproperlyConfigured, '%s is not a valid extension for %s' % (
+                        ext, cls.__name__)
+
+            # Not a string, so take our chances and just try to access "register"
+            else:
+                fn = ext.register
+
+            cls.register_extension(fn)
+            cls._feincms_extensions.add(ext)
 
 
 def create_base_model(inherit_from=models.Model):
-    class Base(inherit_from):
-        """
-        This is the base class for your CMS models.
-        """
+    """
+    This method can  be used to create a FeinCMS base model inheriting from your
+    own custom subclass (f.e. extend ``MPTTModel``). The default is to extend
+    :class:`django.db.models.Model`.
+    """
 
-        content_proxy_class = ContentProxy
+    class Base(inherit_from, ExtensionsMixin):
+        """
+        This is the base class for your CMS models. It knows how to create and
+        manage content types.
+        """
 
         class Meta:
             abstract = True
@@ -233,47 +443,16 @@ def create_base_model(inherit_from=models.Model):
             for template in cls._feincms_templates.values():
                 cls._feincms_all_regions.update(template.regions)
 
-        @classmethod
-        def register_extension(cls, register_fn):
-            """
-            Call the register function of an extension. You must override this
-            if you provide a custom ModelAdmin class and want your extensions to
-            be able to patch stuff in.
-            """
-            register_fn(cls, None)
 
-        @classmethod
-        def register_extensions(cls, *extensions):
-            if not hasattr(cls, '_feincms_extensions'):
-                cls._feincms_extensions = set()
-
-            here = cls.__module__.split('.')[:-1]
-            here_path = '.'.join(here + ['extensions'])
-            common_path = '.'.join(here[:-1] + ['extensions'])
-
-            for ext in extensions:
-                if ext in cls._feincms_extensions:
-                    continue
-
-                if isinstance(ext, basestring):
-                    try:
-                        fn = get_object(ext + '.register')
-                    except ImportError:
-                        try:
-                            fn = get_object('%s.%s.register' % ( here_path, ext ) )
-                        except ImportError:
-                            fn = get_object('%s.%s.register' % ( common_path, ext ) )
-                # Not a string, so take our chances and just try to access "register"
-                else:
-                    fn = ext.register
-
-                cls.register_extension(fn)
-                cls._feincms_extensions.add(ext)
+        #: ``ContentProxy`` class this object uses to collect content blocks
+        content_proxy_class = ContentProxy
 
         @property
         def content(self):
             """
-            Provide a simple interface for getting all content blocks for a region.
+            Instantiate and return a ``ContentProxy``. You can use your own custom
+            ``ContentProxy`` by assigning a different class to the
+            ``content_proxy_class`` member variable.
             """
 
             if not hasattr(self, '_content_proxy'):
@@ -281,64 +460,6 @@ def create_base_model(inherit_from=models.Model):
 
             return self._content_proxy
 
-        def _get_content_types_for_region(self, region):
-            # find all concrete content type tables which have at least one entry for
-            # the current CMS object and region
-            # This method is overridden by a more efficient implementation if
-            # the ct_tracker extension is active.
-
-            from django.core.cache import cache as django_cache
-
-            counts = None
-            ck = None
-            # ???: Should we move the cache_key() method to Base, so we can avoid
-            # the if-it-supports-it dance?
-            if settings.FEINCMS_USE_CACHE and getattr(self, 'cache_key', None):
-                ck = 'CNT-FOR-REGION-' + region.key + '-' + self.cache_key()
-                counts = django_cache.get(ck)
-
-            if counts is None:
-                sql = ' UNION '.join([
-                    'SELECT %d AS ct_idx, COUNT(id) FROM %s WHERE parent_id=%s AND region=%%s' % (
-                        idx,
-                        cls._meta.db_table,
-                        self.pk) for idx, cls in enumerate(self._feincms_content_types)])
-                sql = 'SELECT * FROM ( ' + sql + ' ) AS ct ORDER BY ct_idx'
-
-                from django.db import connection
-                cursor = connection.cursor()
-                cursor.execute(sql, [region.key] * len(self._feincms_content_types))
-
-                counts = [row[1] for row in cursor.fetchall()]
-
-                if ck:
-                    django_cache.set(ck, counts)
-
-            return counts
-
-        def _content_for_region(self, region):
-            """
-            This method is used primarily by the ContentProxy
-            """
-            self._needs_content_types()
-
-            counts = self._get_content_types_for_region(region)
-
-            if not any(counts):
-                return []
-
-            contents = []
-            for idx, cnt in enumerate(counts):
-                if cnt:
-                    # the queryset is evaluated right here, because the content objects
-                    # of different type will have to be sorted into a list according
-                    # to their 'ordering' attribute later
-                    contents += list(
-                        self._feincms_content_types[idx].get_queryset(
-                            Q(parent=self) & Q(region=region.key)))
-
-            return contents
-
         @classmethod
         def _create_content_base(cls):
             """
@@ -439,6 +560,18 @@ def create_base_model(inherit_from=models.Model):
             # list of concrete content types
             cls._feincms_content_types = []
 
+            # list of concrete content types having methods which may be called
+            # before or after rendering the content:
+            #
+            # def process(self, request, **kwargs):
+            #     May return a response early to short-circuit the request-response cycle
+            #
+            # def finalize(self, request, response)
+            #     May modify the response or replace it entirely by returning a new one
+            #
+            cls._feincms_content_types_with_process = []
+            cls._feincms_content_types_with_finalize = []
+
             # list of item editor context processors, will be extended by content types
             if hasattr(cls, 'feincms_item_editor_context_processors'):
                 cls.feincms_item_editor_context_processors = list(cls.feincms_item_editor_context_processors)
@@ -465,6 +598,26 @@ def create_base_model(inherit_from=models.Model):
             pass a list/tuple of region keys as ``regions``. The content type will only
             appear in the corresponding tabs in the item editor.
 
+            If you use two content types with the same name in the same module,
+            name clashes will happen and the content type created first will
+            shadow all subsequent content types. You can work around it by
+            specifying the content type class name using the ``class_name``
+            argument. Please note that this will have an effect on the entries in
+            ``django_content_type``, on ``related_name`` and on the table name
+            used and should therefore not be changed after running ``syncdb`` for
+            the first time.
+
+            Name clashes will also happen if a content type has defined a
+            relationship and you try to register that content type to more than one
+            Base model (in different modules).  Django will raise an error when it
+            tries to create the backward relationship. The solution to that problem
+            is, as shown above, to specify the content type class name with the
+            ``class_name`` argument.
+
+            If you register a content type to more than one Base class, it is
+            recommended to always specify a ``class_name`` when registering it
+            a second time.
+
             You can pass additional keyword arguments to this factory function. These
             keyword arguments will be passed on to the concrete content type, provided
             that it has a ``initialize_type`` classmethod. This is used f.e. in
@@ -531,6 +684,11 @@ def create_base_model(inherit_from=models.Model):
                 attrs)
             cls._feincms_content_types.append(new_type)
 
+            if hasattr(getattr(new_type, 'process', None), '__call__'):
+                cls._feincms_content_types_with_process.append(new_type)
+            if hasattr(getattr(new_type, 'finalize', None), '__call__'):
+                cls._feincms_content_types_with_finalize.append(new_type)
+
             # content types can be limited to a subset of regions
             if not regions:
                 regions = set([region.key for region in cls._feincms_all_regions])
@@ -584,7 +742,7 @@ def create_base_model(inherit_from=models.Model):
             # not knowing about related items, which again causes the bug we had in
             # issue #63 on github:
             #
-            # http://github.com/matthiask/feincms/issues/issue/63/
+            # http://github.com/feincms/feincms/issues/issue/63/
             #
             # Currently, all methods filling up the Model.meta cache start with fill_.
             # We call all these methods upon creation of a new content type to make sure
@@ -630,6 +788,8 @@ def create_base_model(inherit_from=models.Model):
 
         @classmethod
         def _needs_content_types(cls):
+            ensure_completely_loaded()
+
             # Check whether any content types have been created for this base class
             if not hasattr(cls, '_feincms_content_types') or not cls._feincms_content_types:
                 raise ImproperlyConfigured, 'You need to create at least one content type for the %s model.' % (cls.__name__)
@@ -648,10 +808,27 @@ def create_base_model(inherit_from=models.Model):
                     new.save()
 
         def replace_content_with(self, obj):
+            """
+            Replace the content of the current object with content of another.
+
+            Deletes all content blocks and calls ``copy_content_from`` afterwards.
+            """
+
             for cls in self._feincms_content_types:
                 cls.objects.filter(parent=self).delete()
             self.copy_content_from(obj)
 
+        @classmethod
+        def register_with_reversion(cls):
+            if not reversion:
+                raise EnvironmentError("django-reversion is not installed")
+            follow = []
+            for content_type_model in cls._feincms_content_types:
+                related_manager = "%s_set" % content_type_model.__name__.lower()
+                follow.append(related_manager)
+                reversion.register(content_type_model)
+            reversion.register(cls, follow=follow)
+
     return Base
 
 # Legacy support
